from browser import document, window, html, bind
import time
# if the imports to stdlib change, rerun brython --modules in ./ , then rename brython_moduels to brython_corrlinks-text-split.js and then
# move brython_corrlinks-text-split.js to static/


# benchmark: http://www.gutenberg.org/cache/epub/2600/pg2600.txt

def generate_table():
    # for each element in array, create a cell in a 4xN table
    # table y axis labels: MESSAGE,BUTTON TO COPY, CHAR_COUNT, LINE_COUNT

    table = html.TABLE()
    table.class_name = "wide-first-col"
    table.style = {"white-space": "no-wrap"}
    heading = html.TR()
    heading <= html.TH('Message')
    heading <= html.TH('Button to Copy')
    heading <= html.TH('Character Count')
    heading <= html.TH('Line Count')
    heading <= html.TH('Message Efficiency')
    heading <= html.TH('Wasted Cents')
    heading <= html.TH('Message Number')
    table <= heading
    return table


class Output():
    input_str = ''

    cost_per_email = 25 # cost in cents
    char_count = 0
    char_max = 13000
    line_count = 0
    line_max = 99

    output_file_count = 0
    current_file = []

    wasted_cents = 0

    def find_nth(self, haystack, needle, n):
        start = haystack.find(needle)
        while start >= 0 and n > 1:
            start = haystack.find(needle, start + len(needle))
            n -= 1
        return start

    def calculate_wasted_cents(self, message):
        message_len = len(message)
        message_count = message.count('\n')
        wasted_chars = (self.line_max + self.char_max) - (message_len + message_count)
        value_of_wasted_char = self.cost_per_email / (self.line_max + self.char_max)
        wasted_cents = self.cost_per_email - (value_of_wasted_char * (message_len + message_count))
        Output.wasted_cents += wasted_cents
        return round(wasted_cents,2)

    def message_efficiency(self, message):
        message_len = len(message)
        message_count = message.count('\n')
        value = (message_len + message_count) / (self.line_max + self.char_max)
        return '{:.2%}'.format(value)
        pass

    def message_dict(self, message):
        return {'message': message,
                'char_count': len(message) - message.count('\n'),
                'line_count': message.count('\n'),
                'message_efficiency': self.message_efficiency(message),
                'wasted_cents': self.calculate_wasted_cents(message)}

    def parse_message(self):
        """return array of dicts with the values: message, char_count, line_count"""
        output = []
        for start in range(0, len(Output.input_str), self.char_max):
            # message is correct char length, but not correct line length
            message = Output.input_str[start:start + self.char_max]
            sub_message = message
            messages = []
            # if the message is over the max line length we must split it into sub-messages
            if sub_message.count('\n') > self.line_max:
                # sub_message starts as message, but is trimmed down each iteration
                # like a deli slicing machine, the ham is message, and each slice of ham
                # (sliced to line max perfection) is sub_message[:idx] and the rest of ham becomes sub_message[idx:]
                while sub_message.count('\n') > self.line_max:
                    # using nice helper function from SO, we need the line_max^th newline index
                    idx = self.find_nth(sub_message, '\n', self.line_max)
                    messages.append(sub_message[:idx])
                    sub_message = sub_message[idx:]
                # now we need to loop through our sub_messages and append them to output
                for message in messages:
                    output.append(self.message_dict(message))
            # if not, then we just append the message
            else:
                output.append(self.message_dict(message))
        return output

    def draw_output(self):
        total_start = time.time()
        output = document['output']
        start = total_start
        """This will draw a table of text with buttons to copy the text contents"""
        # get rid of empty lines, input_str is a string of the entire input
        Output.input_str = '\n'.join([s for s in Output.input_str.splitlines() if s])  # 0.179 sec benchmark
        print(f'elapsed time to remove uneeded newlines: {time.time() - start}')
        start = time.time()
        result = self.parse_message()
        print(f'elapsed time to parse messages: {time.time() - start}')
        start = time.time()
        table = generate_table()
        print(f'elapsed time to create headings: {time.time() - start}')

        def copy_message_to_clipboard(ev):
            index = (ev.target.parent.previousSibling.children[0])
            index.select()
            index.focus()
            document.execCommand("copy")
            index.blur()

        start = time.time()
        total = len(result)
        for index, record in enumerate(result):
            temp_mess = html.TEXTAREA(record['message'])
            temp_mess.attrs['readonly'] = True
            temp_mess.style = {
                "white-space": "no-wrap",
                "height": "250px",
                "overflow": "scroll",
                "width": "100%"
            }
            cur = html.TR()
            cur <= html.TD(temp_mess)
            cp_btn = html.BUTTON('copy message to clipboard')
            cp_btn.bind("click", copy_message_to_clipboard)
            # I tried making these all go cur <= TD + TD + TD but it was slower than doing it this way
            cur <= html.TD(cp_btn)
            cur <= html.TD(record['char_count'])
            cur <= html.TD(record['line_count'])
            cur <= html.TD(record['message_efficiency'])
            cur <= html.TD(record['wasted_cents'])
            cur <= html.TD(f'{index + 1}/{total}')
            table <= cur
        output <= table
        output <= html.DIV(f'Total wasted value in dollars: $ {round((Output.wasted_cents / 100),2)}')
        print(f'elapsed results loop time: {time.time() - start}')
        print(f'elapsed draw_output time: {time.time() - total_start}')

    def main(self, input_string):
        Output.input_str = input_string
        self.draw_output()


class Input:
    # TODO: have a collapse lines option which will remove all newlines and replace them with ' \ '

    input_str = ''

    def dialog(self, draw, remove, callback, validate=None):
        """Generic function to handle a dialog box, similar to Javascript
        built-in "prompt".
        draw and remove are the functions that draw and remove the dialog box.
        draw() returns a 3-element tuple : the <input> tag, the "Ok" and "Cancel"
        buttons.
        If validate is provided, it must be a callable that is applied to the
        input value:
        - if validate(value) doesn't raise a ValueError, the dialog box is removed
        by calling remove(), and callback(value) is called.
        - if it raises a ValueError, the dialog box is reset.
        """

        inp, btn_ok = draw()

        @bind(btn_ok, "click")
        def click(ev):
            document["input"].clear()
            v = inp.value
            try:
                if validate is not None:
                    validate(v)
                remove()
                callback(v)
            except ValueError:
                self.dialog(draw, remove, callback, validate)

        @bind(inp, "keyup")
        def keyup(ev):
            if ev.keyCode == 13:
                click(ev)

    def draw_dialog(self):
        """Draw the dialog box, with an INPUT field and buttons "Ok" and "Cancel".
        """

        try:
            zone = document["input"]
            zone.clear()

        except KeyError as err:
            zone = html.DIV(id="input")
            document["right"] <= zone

        msg = html.DIV("Enter message you would like split")
        inp = html.TEXTAREA()

        inp.style = {
            "width": "50%",
            "max-height": "200px",
            "height": "200px"
        }
        buttons = html.DIV(style={"text-align": "left"})
        btn_ok = html.BUTTON("Ok")

        buttons <= btn_ok
        zone <= msg + inp + html.BR() + buttons

        # center zone in page
        zone.left = int((window.innerWidth - zone.offsetWidth) / 2)
        zone.top = int(window.innerHeight / 10)

        # set focus on the <input> tag
        inp.focus()

        # return the 3-element tuple expected by dialog()
        return inp, btn_ok

    def remove(self):
        """Remove the dialog box."""
        document["input"].clear()

    def show(self, value):
        """Called when user entered a valid value."""
        document["input"].clear()
        Input.input_str = value
        Output().main(Input.input_str)

    def check_int(self, value):
        """Try to convert value (a string) into an integer. Raises ValueError
        if value is not an integer literal."""
        int(self, value)

    def check_str(self, value):
        str(value)

    def main(self):
        self.dialog(self.draw_dialog,
                    self.remove,
                    validate=self.check_str,
                    callback=self.show)


Input().main()
